#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2008 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;

namespace HeuristicLab.Core {
  public class Environment : ItemBase, IEnvironment {
    private IEnvironment parent;
    private Dictionary<string, string> aliases;

    /// <summary>
    /// Initializes a new instance of <see cref="Environment"/>.
    /// </summary>
    public Environment() {
      aliases = new Dictionary<string, string>();
    }
    /// <summary>
    /// Initializes a new instance of <see cref="Environment"/> with the given <paramref name="parent"/>.
    /// </summary>
    /// <param name="parnet">The parent environment.</param>
    public Environment(IEnvironment parent)
      : this() {
      this.parent = parent;
    }

    /// <summary>
    /// Clones the current instance.
    /// </summary>
    /// <remarks>The parent environment is cloned with the 
    /// <see cref="HeuristicLab.Core.Auxiliary.Clone"/> method of the <see cref="Auxiliary"/> class.</remarks>
    /// <param name="clonedObjects">All already cloned objects. (Needed to avoid cycles.)</param>
    /// <returns>The cloned object as <see cref="AtomicOperation"/>.</returns>
    public override object Clone(IDictionary<Guid, object> clonedObjects) {
      Environment clone = new Environment();
      clonedObjects.Add(Guid, clone);
      clone.parent = (IEnvironment)Auxiliary.Clone(parent, clonedObjects);
      return clone;
    }

    public void AddAlias(string oldName, string newName) {
      aliases.Add(oldName, newName);
    }
    public void RemoveAlias(string name) {
      aliases.Remove(name);
    }

    public string TranslateName(string name) {
      string translatedName = name;
      if (aliases.ContainsKey(translatedName))
        translatedName = aliases[translatedName];
      if (parent != null)
        translatedName = parent.TranslateName(translatedName);
      return translatedName;
    }

    #region Persistence Methods

    /// <summary>
    ///  Saves the current instance as <see cref="XmlNode"/> in the specified <paramref name="document"/>.
    /// </summary>
    /// <remarks>Calls <see cref="HeuristicLab.Core.StorableBase.GetXmlNode"/> of base 
    /// class <see cref="ItemBase"/>. <br/>
    /// The parent environment is saved as child node having the tag name <c>ParentEnvironment</c>.</remarks>
    /// <param name="name">The (tag)name of the <see cref="XmlNode"/>.</param>
    /// <param name="document">The <see cref="XmlDocument"/> where to save the data.</param>
    /// <param name="persistedObjects">The dictionary of all already persisted objects. (Needed to avoid cycles.)</param>
    /// <returns>The saved <see cref="XmlNode"/>.</returns>
    public override XmlNode GetXmlNode(string name, XmlDocument document, IDictionary<Guid, IStorable> persistedObjects) {
      XmlNode node = base.GetXmlNode(name, document, persistedObjects);
      node.AppendChild(PersistenceManager.Persist("ParentEnvironment", parent, document, persistedObjects));
      return node;
    }
    /// <summary>
    /// Loads the persisted operation from the specified <paramref name="node"/>.
    /// </summary>
    /// <remarks>Calls <see cref="HeuristicLab.Core.StorableBase.Populate"/> of base class 
    /// <see cref="ItemBase"/>.<br/>
    /// The parent environment must be saved as a child node with the tag name <c>ParentEnvironment</c>
    /// (see <see cref="GetXmlNode"/>).</remarks>
    /// <param name="node">The <see cref="XmlNode"/> where the operation is saved.</param>
    /// <param name="restoredObjects">A dictionary of all already restored objects. (Needed to avoid cycles.)</param>
    public override void Populate(XmlNode node, IDictionary<Guid, IStorable> restoredObjects) {
      base.Populate(node, restoredObjects);
      parent = (IEnvironment)PersistenceManager.Restore(node.SelectSingleNode("ParentEnvironment"), restoredObjects);
    }
    #endregion
  }
}
